@@ -5,7 +5,7 @@
 
 """
 USAGE: %(program)s LANGUAGE
-    Process the repository, accepting articles in LANGUAGE (or 'any'). 
+    Process the repository, accepting articles in LANGUAGE (or 'any').
     Store the word co-occurence matrix and id mappings, which are needed for subsequent processing.
 
 Example: ./gensim_build.py eng
@@ -21,6 +21,8 @@ import re
 from gensim.corpora import sources, dmlcorpus
 
 
+PREFIX = 'dmlcz'
+
 AT_HOME = False
 
 if AT_HOME:
@@ -29,11 +31,11 @@ if AT_HOME:
                    sources.DmlSource('numdam', '/Users/kofola/workspace/dml/data/numdam/'),
                    sources.ArxmlivSource('arxmliv', '/Users/kofola/workspace/dml/data/arxmliv/'),
                    ]
-    
+
 #    SOURCE_LIST = [
 #                   sources.DmlCzSource('dmlcz', '/Users/kofola/workspace/dml/data/dmlcz/CzechMathJ'),
 #                   ]
-    
+
     RESULT_DIR = '/Users/kofola/workspace/dml/data/results'
 
 else:
@@ -43,7 +45,7 @@ else:
                    sources.DmlSource('numdam', '/data/dmlcz/data/numdam'),
                    sources.ArxmlivSource('arxmliv', '/data/dmlcz/data/arxmliv'),
                    ]
-    
+
     RESULT_DIR = '/data/dmlcz/xrehurek/results'
 
 
@@ -51,16 +53,16 @@ def buildDmlCorpus(config):
     dml = dmlcorpus.DmlCorpus()
     dml.processConfig(config, shuffle = True)
     dml.buildDictionary()
-    dml.dictionary.filterExtremes(noBelow = 5, noAbove = 0.3) # ignore too (in)frequent words
-    
-    dml.save(config.resultFile('.pkl')) # save the mappings as binary data (actual documents are not saved, only their uris) 
+    dml.dictionary.filterExtremes(noBelow=5, noAbove=0.3) # ignore too (in)frequent words
+
+    dml.save(config.resultFile('.pkl')) # save the mappings as binary data (actual documents are not saved, only their URIs)
     dml.saveAsText() # save id mappings and documents as text data (matrix market format)
     return dml
 
 
 if __name__ == '__main__':
-    logging.basicConfig(format = '%(asctime)s : %(levelname)s : %(message)s')
-    logging.root.setLevel(level = logging.INFO) 
+    logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s')
+    logging.root.setLevel(level=logging.INFO)
     logging.info("running %s" % ' '.join(sys.argv))
 
     program = os.path.basename(sys.argv[0])
@@ -70,12 +72,12 @@ if __name__ == '__main__':
         print globals()['__doc__'] % locals()
         sys.exit(1)
     language = sys.argv[1]
-    
+
     # construct the config, which holds information about sources, data file filenames etc.
-    config = dmlcorpus.DmlConfig('gensim_%s' % language, resultDir = RESULT_DIR, acceptLangs = [language])
+    config = dmlcorpus.DmlConfig('%s_%s' % (PREFIX, language), resultDir=RESULT_DIR, acceptLangs=[language])
     for source in SOURCE_LIST:
         config.addSource(source)
     buildDmlCorpus(config)
-    
+
     logging.info("finished running %s" % program)
 
