@@ -16,15 +16,31 @@ import os.path
 import tempfile
 
 import numpy
+from nose.tools import raises
 
-from gensim.corpora import mmcorpus
-from gensim.models import lsimodel, ldamodel, tfidfmodel, rpmodel
+from gensim.corpora import mmcorpus, Dictionary
+from gensim.models import lsimodel, ldamodel, tfidfmodel, rpmodel, logentropy_model
 from gensim import matutils
 
 
 module_path = os.path.dirname(__file__) # needed because sample data files are located in the same folder
+datapath = lambda fname: os.path.join(module_path, 'test_data', fname)
 
-logging.basicConfig(format = '%(asctime)s : %(levelname)s : %(message)s', level=logging.WARNING)
+logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.WARNING)
+
+
+# set up vars used in testing ("Deerwester" from the web tutorial)
+texts = [['human', 'interface', 'computer'],
+ ['survey', 'user', 'computer', 'system', 'response', 'time'],
+ ['eps', 'user', 'interface', 'system'],
+ ['system', 'human', 'system', 'eps'],
+ ['user', 'response', 'time'],
+ ['trees'],
+ ['graph', 'trees'],
+ ['graph', 'minors', 'trees'],
+ ['graph', 'minors', 'survey']]
+dictionary = Dictionary(texts)
+corpus = [dictionary.doc2bow(text) for text in texts]
 
 
 def testfile():
@@ -34,67 +50,67 @@ def testfile():
 
 class TestLsiModel(unittest.TestCase):
     def setUp(self):
-        self.corpus = mmcorpus.MmCorpus(os.path.join(module_path, 'testcorpus.mm'))
-    
+        self.corpus = mmcorpus.MmCorpus(datapath('testcorpus.mm'))
+
     def testTransform(self):
         # create the transformation model
         model = lsimodel.LsiModel(self.corpus, numTopics=2)
-        
+
         # make sure the decomposition is enough accurate
         u, s, vt = numpy.linalg.svd(matutils.corpus2dense(self.corpus, self.corpus.numTerms), full_matrices=False)
         self.assertTrue(numpy.allclose(s[:2], model.projection.s)) # singular values must match
-        
+
         # transform one document
         doc = list(self.corpus)[0]
         transformed = model[doc]
         vec = matutils.sparse2full(transformed, 2) # convert to dense vector, for easier equality tests
-        
+
         expected = numpy.array([-0.6594664, 0.142115444]) # scaled LSI version
         # expected = numpy.array([-0.1973928, 0.05591352]) # non-scaled LSI version
         self.assertTrue(numpy.allclose(abs(vec), abs(expected))) # transformed entries must be equal up to sign
-    
-    
+
+
     def testOnlineTransform(self):
         corpus = list(self.corpus)
         doc = corpus[0] # use the corpus' first document for testing
-        
+
         # create the transformation model
-        model2 = lsimodel.LsiModel(corpus = corpus, numTopics = 5) # compute everything at once
-        model = lsimodel.LsiModel(corpus = None, id2word = model2.id2word, numTopics = 5) # start with no documents, we will add then later
-        
+        model2 = lsimodel.LsiModel(corpus=corpus, numTopics=5) # compute everything at once
+        model = lsimodel.LsiModel(corpus=None, id2word=model2.id2word, numTopics=5) # start with no documents, we will add then later
+
         # train model on a single document
         model.addDocuments([corpus[0]])
         model.printDebug()
-        
+
         # transform the testing document with this partial transformation
         transformed = model[doc]
         vec = matutils.sparse2full(transformed, model.numTopics) # convert to dense vector, for easier equality tests
         expected = numpy.array([-1.73205078, 0.0, 0.0, 0.0, 0.0]) # scaled LSI version
-        self.assertTrue(numpy.allclose(abs(vec), abs(expected), atol = 1e-6)) # transformed entries must be equal up to sign
-        
+        self.assertTrue(numpy.allclose(abs(vec), abs(expected), atol=1e-6)) # transformed entries must be equal up to sign
+
         # train on another 4 documents
-        model.addDocuments(corpus[1:5], chunks = 2) # train in chunks of 2 documents, for the lols
+        model.addDocuments(corpus[1:5], chunks=2) # train in chunks of 2 documents, for the lols
         model.printDebug()
-        
+
         # transform a document with this partial transformation
         transformed = model[doc]
         vec = matutils.sparse2full(transformed, model.numTopics) # convert to dense vector, for easier equality tests
-        expected = numpy.array([-0.66493785, -0.28314203, -1.56376302,  0.05488682,  0.17123269]) # scaled LSI version
-        m2 = lsimodel.LsiModel(corpus = list(corpus)[:5], numTopics = 5)
-        self.assertTrue(numpy.allclose(abs(vec), abs(expected), atol = 1e-6)) # transformed entries must be equal up to sign
-        
+        expected = numpy.array([-0.66493785, -0.28314203, -1.56376302, 0.05488682, 0.17123269]) # scaled LSI version
+        m2 = lsimodel.LsiModel(corpus=list(corpus)[:5], numTopics=5)
+        self.assertTrue(numpy.allclose(abs(vec), abs(expected), atol=1e-6)) # transformed entries must be equal up to sign
+
         # train on the rest of documents
         model.addDocuments(corpus[5:])
         model.printDebug()
-        
+
         # make sure the final transformation is the same as if we had decomposed the whole corpus at once
         vec1 = matutils.sparse2full(model[doc], model.numTopics)
         vec2 = matutils.sparse2full(model2[doc], model2.numTopics)
-        self.assertTrue(numpy.allclose(abs(vec1), abs(vec2), atol = 1e-6)) # the two LSI representations must equal up to sign
+        self.assertTrue(numpy.allclose(abs(vec1), abs(vec2), atol=1e-5)) # the two LSI representations must equal up to sign
+
 
-    
     def testPersistence(self):
-        model = lsimodel.LsiModel(self.corpus, numTopics = 2)
+        model = lsimodel.LsiModel(self.corpus, numTopics=2)
         model.save(testfile())
         model2 = lsimodel.LsiModel.load(testfile())
         self.assertEqual(model.numTopics, model2.numTopics)
@@ -107,24 +123,24 @@ class TestLsiModel(unittest.TestCase):
 
 class TestRpModel(unittest.TestCase):
     def setUp(self):
-        self.corpus = mmcorpus.MmCorpus(os.path.join(module_path, 'testcorpus.mm'))
-    
+        self.corpus = mmcorpus.MmCorpus(datapath('testcorpus.mm'))
+
     def testTransform(self):
         # create the transformation model
         numpy.random.seed(13) # HACK; set fixed seed so that we always get the same random matrix (and can compare against expected results)
-        model = rpmodel.RpModel(self.corpus, numTopics = 2)
-        
+        model = rpmodel.RpModel(self.corpus, numTopics=2)
+
         # transform one document
         doc = list(self.corpus)[0]
         transformed = model[doc]
         vec = matutils.sparse2full(transformed, 2) # convert to dense vector, for easier equality tests
-        
+
         expected = numpy.array([-0.70710677, 0.70710677])
         self.assertTrue(numpy.allclose(vec, expected)) # transformed entries must be equal up to sign
-        
-    
+
+
     def testPersistence(self):
-        model = rpmodel.RpModel(self.corpus, numTopics = 2)
+        model = rpmodel.RpModel(self.corpus, numTopics=2)
         model.save(testfile())
         model2 = rpmodel.RpModel.load(testfile())
         self.assertEqual(model.numTopics, model2.numTopics)
@@ -136,23 +152,34 @@ class TestRpModel(unittest.TestCase):
 
 class TestLdaModel(unittest.TestCase):
     def setUp(self):
-        self.corpus = mmcorpus.MmCorpus(os.path.join(module_path, 'testcorpus.mm'))
-    
+        self.corpus = mmcorpus.MmCorpus(datapath('testcorpus.mm'))
+
     def testTransform(self):
-        # create the transformation model
-        model = ldamodel.LdaModel(self.corpus, numTopics=2, passes=100) # 100 passes ought to be enough to converge
-        
-        # transform one document
-        doc = list(self.corpus)[0]
-        transformed = model[doc]
-        
-        vec = matutils.sparse2full(transformed, 2) # convert to dense vector, for easier equality tests
-        expected = [0.87, 0.13]
-        assert numpy.allclose(sorted(vec), sorted(expected), atol=0.01)  # must contain the same values, up to re-ordering
-        
-    
+        passed = False
+        # sometimes, LDA training gets stuck at a local minimum
+        # in that case try re-training the model from scratch, hoping for a 
+        # better random initialization
+        for i in xrange(5): # restart at most 5 times
+            # create the transformation model
+            model = ldamodel.LdaModel(id2word=dictionary, numTopics=2, passes=100)
+            model.update(corpus)
+            
+            # transform one document
+            doc = list(corpus)[0]
+            transformed = model[doc]
+            
+            vec = matutils.sparse2full(transformed, 2) # convert to dense vector, for easier equality tests
+            expected = [0.049, 0.951]
+            passed = numpy.allclose(sorted(vec), sorted(expected), atol=1e-2) # must contain the same values, up to re-ordering
+            if passed:
+                break
+            logging.warning("LDA failed to converge on attempt %i (got %s, expected %s)" % 
+                            (i, sorted(vec), sorted(expected)))
+        self.assertTrue(passed)
+
+
     def testPersistence(self):
-        model = ldamodel.LdaModel(self.corpus, numTopics = 2)
+        model = ldamodel.LdaModel(self.corpus, numTopics=2)
         model.save(testfile())
         model2 = ldamodel.LdaModel.load(testfile())
         self.assertEqual(model.numTopics, model2.numTopics)
@@ -164,22 +191,38 @@ class TestLdaModel(unittest.TestCase):
 
 class TestTfidfModel(unittest.TestCase):
     def setUp(self):
-        self.corpus = mmcorpus.MmCorpus(os.path.join(module_path, 'testcorpus.mm'))
-    
+        self.corpus = mmcorpus.MmCorpus(datapath('testcorpus.mm'))
+
     def testTransform(self):
         # create the transformation model
-        model = tfidfmodel.TfidfModel(self.corpus, normalize = True)
-        
+        model = tfidfmodel.TfidfModel(self.corpus, normalize=True)
+
         # transform one document
         doc = list(self.corpus)[0]
         transformed = model[doc]
-        
+
         expected =  [(0, 0.57735026918962573), (1, 0.57735026918962573), (2, 0.57735026918962573)]
         self.assertTrue(numpy.allclose(transformed, expected))
+
+
+    def testInit(self):
+        # create the transformation model by analyzing a corpus 
+        # uses the global `corpus`!
+        model1 = tfidfmodel.TfidfModel(corpus)
         
+        # make sure the dfs<->idfs transformation works
+        dfs = tfidfmodel.idfs2dfs(model1.idfs, len(corpus))
+        self.assertEqual(dfs, dictionary.dfs)
+        self.assertEqual(model1.idfs, tfidfmodel.dfs2idfs(dfs, len(corpus)))
+        
+        # create the transformation model by directly supplying a term->docfreq 
+        # mapping from the global var `dictionary`.
+        model2 = tfidfmodel.TfidfModel(dictionary=dictionary)
+        self.assertEqual(model1.idfs, model2.idfs)
+    
     
     def testPersistence(self):
-        model = tfidfmodel.TfidfModel(self.corpus, normalize = True)
+        model = tfidfmodel.TfidfModel(self.corpus, normalize=True)
         model.save(testfile())
         model2 = tfidfmodel.TfidfModel.load(testfile())
         self.assertTrue(model.idfs == model2.idfs)
@@ -188,6 +231,48 @@ class TestTfidfModel(unittest.TestCase):
 #endclass TestTfidfModel
 
 
+class TestLogEntropyModel(unittest.TestCase):
+    def setUp(self):
+        self.corpus_small = mmcorpus.MmCorpus(datapath('test_corpus_small.mm'))
+        self.corpus_ok = mmcorpus.MmCorpus(datapath('test_corpus_ok.mm'))
+
+    @raises(ValueError)
+    def test_corpus_validity(self):
+        """
+        check whether the corpus is valid
+
+        words that appear in only one context cause trouble in the log entropy
+        normalization and should therefore be filtered out.
+        --> context diversity should be > 1
+        """
+        logentropy_model.LogEntropyModel(self.corpus_small)
+
+
+    def testTransform(self):
+        # create the transformation model
+        model = logentropy_model.LogEntropyModel(self.corpus_ok, normalize=False)
+
+        # transform one document
+        doc = list(self.corpus_ok)[0]
+        transformed = model[doc]
+        expected =  [(0, 0.056633012265132537),
+                     (1, 0.024757785476437949),
+                     (3, 0.62707564002906502)]
+        self.assertTrue(numpy.allclose(transformed, expected))
+
+
+
+    def testPersistence(self):
+        model = logentropy_model.LogEntropyModel(self.corpus_ok, normalize=True)
+        model.save(testfile())
+        model2 = logentropy_model.LogEntropyModel.load(testfile())
+        self.assertTrue(model.entr == model2.entr)
+        tstvec = []
+        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec]))
+#endclass TestLogEntropyModel
+
+
+
 if __name__ == '__main__':
     logging.root.setLevel(logging.WARNING)
     unittest.main()
