@@ -34,7 +34,7 @@ RE_HTML_ENTITY = re.compile(r'&(#?)(x?)(\w+);', re.UNICODE)
 def synchronous(tlockname):
     """
     A decorator to place an instance-based lock around a method.
-    
+
     Adapted from http://code.activestate.com/recipes/577105-synchronization-decorator-for-class-methods/
     """
     def _synched(func):
@@ -51,14 +51,14 @@ def synchronous(tlockname):
 def deaccent(text):
     """
     Remove accentuation from the given string. Input text is either a unicode string or utf8 encoded bytestring.
-    
+
     Return input string with accents removed, as unicode.
-    
+
     >>> deaccent("Šéf chomutovských komunistů dostal poštou bílý prášek")
     u'Sef chomutovskych komunistu dostal postou bily prasek'
     """
     if not isinstance(text, unicode):
-        text = unicode(text, 'utf8') # assume utf8 for byte strings, use default (strict) error handling 
+        text = unicode(text, 'utf8') # assume utf8 for byte strings, use default (strict) error handling
     norm = unicodedata.normalize("NFD", text)
     result = u''.join(ch for ch in norm if unicodedata.category(ch) != 'Mn')
     return unicodedata.normalize("NFC", result)
@@ -66,14 +66,14 @@ def deaccent(text):
 
 def tokenize(text, lowercase=False, deacc=False, errors="strict", toLower=False, lower=False):
     """
-    Iteratively yield tokens as unicode strings, optionally also lowercasing them 
+    Iteratively yield tokens as unicode strings, optionally also lowercasing them
     and removing accent marks.
-    
+
     Input text may be either unicode or utf8-encoded byte string.
 
-    The tokens on output are maximal contiguous sequences of alphabetic 
+    The tokens on output are maximal contiguous sequences of alphabetic
     characters (no digits!).
-    
+
     >>> list(tokenize('Nic nemůže letět rychlostí vyšší, než 300 tisíc kilometrů za sekundu!', deacc = True))
     [u'Nic', u'nemuze', u'letet', u'rychlosti', u'vyssi', u'nez', u'tisic', u'kilometru', u'za', u'sekundu']
     """
@@ -96,10 +96,10 @@ def toUtf8(text):
 
 class SaveLoad(object):
     """
-    Objects which inherit from this class have save/load functions, which un/pickle 
+    Objects which inherit from this class have save/load functions, which un/pickle
     them to disk.
-    
-    This uses cPickle for de/serializing, so objects must not contains unpicklable 
+
+    This uses cPickle for de/serializing, so objects must not contains unpicklable
     attributes, such as lambda functions etc.
     """
     @classmethod
@@ -108,17 +108,14 @@ class SaveLoad(object):
         Load a previously saved object from file (also see `save`).
         """
         logger.info("loading %s object from %s" % (cls.__name__, fname))
-        return cPickle.load(open(fname, 'rb')) # 'b' for binary, needed on Windows
-
+        return unpickle(fname)
 
     def save(self, fname):
         """
         Save the object to file via pickling (also see `load`).
         """
         logger.info("saving %s object to %s" % (self.__class__.__name__, fname))
-        f = open(fname, 'wb')
-        cPickle.dump(self, f, protocol=-1) # -1 to use the highest available protocol, for efficiency
-        f.close()
+        pickle(self, fname)
 #endclass SaveLoad
 
 
@@ -129,54 +126,54 @@ def identity(p):
 def getMaxId(corpus):
     """
     Return highest feature id that appears in the corpus.
-    
+
     For empty corpora (no features at all), return -1.
     """
     maxId = -1
     for document in corpus:
-        maxId = max(maxId, max([-1] + [fieldId for fieldId, _ in document])) # [-1] to avoid exceptions from max(empty) 
+        maxId = max(maxId, max([-1] + [fieldId for fieldId, _ in document])) # [-1] to avoid exceptions from max(empty)
     return maxId
 
 
 class FakeDict(object):
     """
-    Objects of this class act as dictionaries that map integer->str(integer), for 
+    Objects of this class act as dictionaries that map integer->str(integer), for
     a specified range of integers <0, numTerms).
-    
+
     This is meant to avoid allocating real dictionaries when numTerms is huge, which
     is a waste of memory.
     """
     def __init__(self, numTerms):
         self.numTerms = numTerms
-    
+
 
     def __str__(self):
         return "FakeDict(numTerms=%s)" % self.numTerms
-    
-    
+
+
     def __getitem__(self, val):
         if 0 <= val < self.numTerms:
             return str(val)
-        raise ValueError("internal id out of bounds (%s, expected <0..%s))" % 
+        raise ValueError("internal id out of bounds (%s, expected <0..%s))" %
                          (val, self.numTerms))
-    
+
     def iteritems(self):
         for i in xrange(self.numTerms):
             yield i, str(i)
-    
+
     def keys(self):
         """
-        Override the dict.keys() function, which is used to determine the maximum 
+        Override the dict.keys() function, which is used to determine the maximum
         internal id of a corpus = the vocabulary dimensionality.
-        
-        HACK: To avoid materializing the whole `range(0, self.numTerms)`, this returns 
+
+        HACK: To avoid materializing the whole `range(0, self.numTerms)`, this returns
         `[self.numTerms - 1]` only.
         """
         return [self.numTerms - 1]
-    
+
     def __len__(self):
         return self.numTerms
-    
+
     def get(self, val, default=None):
         if 0 <= val < self.numTerms:
             return str(val)
@@ -187,9 +184,9 @@ def dictFromCorpus(corpus):
     """
     Scan corpus for all word ids that appear in it, then construct and return a mapping
     which maps each ``wordId -> str(wordId)``.
-    
-    This function is used whenever *words* need to be displayed (as opposed to just 
-    their ids) but no wordId->word mapping was provided. The resulting mapping 
+
+    This function is used whenever *words* need to be displayed (as opposed to just
+    their ids) but no wordId->word mapping was provided. The resulting mapping
     only covers words actually used in the corpus, up to the highest wordId found.
     """
     numTerms = 1 + getMaxId(corpus)
@@ -202,11 +199,11 @@ def isCorpus(obj):
     Check whether `obj` is a corpus. Return (is_corpus, new) 2-tuple, where
     `new is obj` if `obj` was an iterable, or `new` yields the same sequence as
     `obj` if it was an iterator.
-    
+
     `obj` is a corpus if it supports iteration over documents, where a document
     is in turn anything that acts as a sequence of 2-tuples (int, float).
-    
-    Note: An "empty" corpus (empty input sequence) is ambiguous, so in this case the 
+
+    Note: An "empty" corpus (empty input sequence) is ambiguous, so in this case the
     result is forcefully defined as `is_corpus=False`.
     """
     try:
@@ -217,12 +214,12 @@ def isCorpus(obj):
     try:
         if hasattr(obj, 'next'):
             # the input is an iterator object, meaning once we call next()
-            # that element could be gone forever. we must be careful to put 
+            # that element could be gone forever. we must be careful to put
             # whatever we retrieve back again
             doc1 = obj.next()
             obj = itertools.chain([doc1], obj)
         else:
-            doc1 = iter(obj).next() # empty corpus is resolved to False here 
+            doc1 = iter(obj).next() # empty corpus is resolved to False here
         if len(doc1) == 0: # sparse documents must have a __len__ function (list, tuple...)
             return True, obj # the first document is empty=>assume this is a corpus
         id1, val1 = iter(doc1).next() # if obj is a vector, it resolves to False here
@@ -230,16 +227,16 @@ def isCorpus(obj):
     except:
         return False, obj
     return True, obj
-    
+
 
 
 def get_my_ip():
     """
     Try to obtain our external ip (from the pyro nameserver's point of view)
-    
-    This tries to sidestep the issue of bogus `/etc/hosts` entries and other 
+
+    This tries to sidestep the issue of bogus `/etc/hosts` entries and other
     local misconfigurations, which often mess up hostname resolution.
-    
+
     If all else fails, fall back to simple `socket.gethostbyname()` lookup.
     """
     import socket
@@ -266,15 +263,15 @@ def get_my_ip():
 class RepeatCorpus(SaveLoad):
     """
     Used in the tutorial on distributed computing and likely not useful anywhere else.
-    
+
     """
     def __init__(self, corpus, reps):
         """
         Wrap a `corpus` as another corpus of length `reps`. This is achieved by
         repeating documents from `corpus` over and over again, until the requested
-        length `len(result)==reps` is reached. Repetition is done 
-        on-the-fly=efficiently, via `itertools`. 
-        
+        length `len(result)==reps` is reached. Repetition is done
+        on-the-fly=efficiently, via `itertools`.
+
         >>> corpus = [[(1, 0.5)], []] # 2 documents
         >>> list(RepeatCorpus(corpus, 5)) # repeat 2.5 times to get 5 documents
         [[(1, 0.5)], [], [(1, 0.5)], [], [(1, 0.5)]]
@@ -282,7 +279,7 @@ class RepeatCorpus(SaveLoad):
         """
         self.corpus = corpus
         self.reps = reps
-    
+
     def __iter__(self):
         return itertools.islice(itertools.cycle(self.corpus), self.reps)
 
@@ -290,9 +287,9 @@ class RepeatCorpus(SaveLoad):
 def decode_htmlentities(text):
     """
     Decode HTML entities in text, coded as hex, decimal or named.
-    
+
     Adapted from http://github.com/sku/python-twitter-ircbot/blob/321d94e0e40d0acc92f5bf57d126b57369da70de/html_decode.py
-    
+
     >>> u = u'E tu vivrai nel terrore - L&#x27;aldil&#xE0; (1981)'
     >>> print decode_htmlentities(u).encode('UTF-8')
     E tu vivrai nel terrore - L'aldilà (1981)
@@ -300,7 +297,7 @@ def decode_htmlentities(text):
     l'eau
     >>> print decode_htmlentities("foo &lt; bar")
     foo < bar
-    
+
     """
     def substitute_entity(match):
         ent = match.group(3)
@@ -315,11 +312,11 @@ def decode_htmlentities(text):
         else:
             # they were using a name
             cp = n2cp.get(ent)
-            if cp: 
+            if cp:
                 return unichr(cp)
             else:
                 return match.group()
-    
+
     try:
         return RE_HTML_ENTITY.sub(substitute_entity, text)
     except:
@@ -331,17 +328,17 @@ def decode_htmlentities(text):
 def chunkize_serial(corpus, chunks):
     """
     Split a stream of values into smaller chunks.
-    Each chunk is of length `chunks`, except the last one which may be smaller. 
+    Each chunk is of length `chunks`, except the last one which may be smaller.
     A once-only input stream (`corpus` from a generator) is ok.
-    
+
     Chunking is done efficiently with itertools, so that only one chunk at a time
     is realized in memory (the whole `corpus` may be larger than RAM).
-    
+
     >>> for chunk in chunkize(xrange(10), 4): print chunk
     [0, 1, 2, 3]
     [4, 5, 6, 7]
     [8, 9]
-      
+
     """
     chunker = itertools.groupby(enumerate(corpus), key = lambda (docno, doc): docno / chunks)
     for chunk_no, (key, group) in enumerate(chunker):
@@ -360,14 +357,17 @@ class InputQueue(threading.Thread):
         self.maxsize = maxsize
         self.corpus = corpus
         self.chunks = chunks
-    
+
     def run(self):
         import numpy # don't clutter the global namespace with a dependency on numpy
         for chunk in chunkize_serial(self.corpus, self.chunks):
             # HACK XXX convert documents to numpy arrays, to save memory.
-            # makes no difference if docs are plain standard list of 2-tuples etc, 
+            # makes no difference if docs are plain standard list of 2-tuples etc,
             # but what if they are more complex, user-defined objects?
+            # this also gives a scipy warning at runtime:
+            # "UserWarning: indices array has non-integer dtype (float64)"
             chunk = [numpy.asarray(doc) for doc in chunk]
+            logger.debug("pre-prepared another chunk (qsize=%s)" % self.q.qsize())
             self.q.put(chunk, block=True)
 #endclass InputQueue
 
@@ -375,32 +375,32 @@ class InputQueue(threading.Thread):
 def chunkize(corpus, chunks, maxsize=0):
     """
     Split a stream of values into smaller chunks.
-    Each chunk is of length `chunks`, except the last one which may be smaller. 
+    Each chunk is of length `chunks`, except the last one which may be smaller.
     A once-only input stream (`corpus` from a generator) is ok, chunking is done
     efficiently via itertools.
-    
-    If `maxsize > 1`, don't wait idly in between successive chunk `yields`, but 
-    rather keep filling a short queue (of size at most `maxsize`) with forthcoming 
-    chunks in advance. This is realized by starting a separate thread, and is 
-    meant to reduce I/O delays, which can be significant when `corpus` comes 
+
+    If `maxsize > 1`, don't wait idly in between successive chunk `yields`, but
+    rather keep filling a short queue (of size at most `maxsize`) with forthcoming
+    chunks in advance. This is realized by starting a separate thread, and is
+    meant to reduce I/O delays, which can be significant when `corpus` comes
     from a slow medium (like harddisk).
-    
-    If `maxsize==0`, don't fool around with threads and simply yield the chunks 
+
+    If `maxsize==0`, don't fool around with threads and simply yield the chunks
     via `chunkize_serial()` (no I/O optimizations).
-    
+
     >>> for chunk in chunkize(xrange(10), 4): print chunk
     [0, 1, 2, 3]
     [4, 5, 6, 7]
     [8, 9]
-      
+
     """
     assert chunks > 0
-    
+
     if maxsize > 0:
         q = Queue(maxsize=maxsize)
         thread = InputQueue(q, corpus, chunks, maxsize=maxsize)
         thread.start()
-        
+
         while thread.isAlive() or not q.empty():
             yield q.get(block=True)
     else:
@@ -408,3 +408,41 @@ def chunkize(corpus, chunks, maxsize=0):
             yield chunk
 
 
+def pickle(obj, fname, protocol=-1):
+    """Pickle object `obj` to file `fname`."""
+    with open(fname, 'wb') as fout: # 'b' for binary, needed on Windows
+        cPickle.dump(obj, fout, protocol=protocol)
+
+
+def unpickle(fname):
+    """Load pickled object from `fname`"""
+    return cPickle.load(open(fname, 'rb'))
+
+
+def revdict(d):
+    """
+    Reverse a dictionary mapping. 
+    
+    When two keys map to the same value, only one of them will be kept in the 
+    result (which one is kept is arbitrary)."""
+    return dict((v, k) for (k, v) in d.iteritems())
+
+
+def toptexts(query, texts, index, n=10):
+    """
+    Debug fnc to help inspect the top `n` most similar documents (according to a 
+    similarity index `index`), to see if they are actually related to the query.
+    
+    `texts` is any object that can return something insightful for each document 
+    via `texts[docid]`, such as its fulltext or snippet.
+    
+    Return a list of 3-tuples (docid, doc's similarity to the query, texts[docid]).
+    """
+    sims = index[query] # perform a similarity query against the corpus
+    sims = sorted(enumerate(sims), key=lambda item: -item[1])
+    
+    result = []
+    for topid, topcosine in sims[:n]: # only consider top-n most similar docs
+        result.append((topid, topcosine, texts[topid]))
+    return result
+
